@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008-2009, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008-2009, Nokia <ivan.frade@nokia.com>
+ * Copyright (C) 2010, Amit Aggarwal <amitcs06@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,14 +26,12 @@
 #include <glib.h>
 #include <poppler.h>
 
-#include <libtracker-common/tracker-statement-list.h>
-#include <libtracker-common/tracker-ontology.h>
-#include <libtracker-common/tracker-type-utils.h>
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-utils.h>
-#include <libtracker-common/tracker-type-utils.h>
+
+#include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-main.h"
-#include "tracker-xmp.h"
 
 typedef struct {
 	gchar *title;
@@ -44,6 +43,7 @@ typedef struct {
 } PDFData;
 
 static void extract_pdf (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
                          TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
@@ -51,6 +51,135 @@ static TrackerExtractData data[] = {
 	{ NULL, NULL }
 };
 
+static void
+read_toc (PopplerIndexIter  *index,
+          GString          **toc)
+{
+        if (!index) {
+	        return;
+        }
+
+        if (!*toc) {
+	        *toc = g_string_new ("");
+        }
+
+        do {
+                PopplerAction *action;
+                PopplerIndexIter *iter;
+
+                action = poppler_index_iter_get_action (index);
+
+                if (!action) {
+                        continue;
+                }
+
+                switch (action->type) {
+                case POPPLER_ACTION_GOTO_DEST: {
+                        PopplerActionGotoDest *ag = (PopplerActionGotoDest*) action;
+                        PopplerDest *agd = ag->dest;
+
+                        if (!tracker_is_empty_string (ag->title)) {
+	                        g_string_append_printf (*toc, "%s ", ag->title);
+                        }
+
+                        if (!tracker_is_empty_string (agd->named_dest)) {
+	                        g_string_append_printf (*toc, "%s ", agd->named_dest);
+                        }
+
+                        break;
+                }
+
+                case POPPLER_ACTION_LAUNCH: {
+                        PopplerActionLaunch *al = (PopplerActionLaunch*) action;
+
+                        if (!tracker_is_empty_string (al->title)) {
+	                        g_string_append_printf (*toc, "%s ", al->title);
+                        }
+
+                        if (!tracker_is_empty_string (al->file_name)) {
+	                        g_string_append_printf (*toc, "%s ", al->file_name);
+                        }
+
+                        if (!tracker_is_empty_string (al->params)) {
+	                        g_string_append_printf (*toc, "%s ", al->params);
+                        }
+
+                        break;
+                }
+
+                case POPPLER_ACTION_URI: {
+                        PopplerActionUri *au = (PopplerActionUri*) action;
+
+                        if (!tracker_is_empty_string (au->uri)) {
+	                        g_string_append_printf (*toc, "%s ", au->uri);
+                        }
+
+                        break;
+                }
+
+                case POPPLER_ACTION_NAMED: {
+                        PopplerActionNamed *an = (PopplerActionNamed*) action;
+
+                        if (!tracker_is_empty_string (an->title)) {
+	                        g_string_append_printf (*toc, "%s, ", an->title);
+                        }
+
+                        if (!tracker_is_empty_string (an->named_dest)) {
+                                g_string_append_printf (*toc, "%s ", an->named_dest);
+                        }
+
+                        break;
+                }
+
+                case POPPLER_ACTION_MOVIE: {
+                        PopplerActionNamed *am = (PopplerActionNamed*) action;
+
+                        if (!tracker_is_empty_string (am->title)) {
+	                        g_string_append_printf (*toc, "%s ", am->title);
+                        }
+
+                        break;
+                }
+
+                case POPPLER_ACTION_NONE:
+                case POPPLER_ACTION_UNKNOWN:
+                case POPPLER_ACTION_GOTO_REMOTE:
+                        /* Do nothing */
+                        break;
+                }
+
+                iter = poppler_index_iter_get_child (index);
+                read_toc (iter, toc);
+        } while (poppler_index_iter_next (index));
+
+        poppler_index_iter_free (index);
+}
+
+static void
+read_outline (PopplerDocument      *document,
+              TrackerSparqlBuilder *metadata)
+{
+        PopplerIndexIter *index;
+        GString *toc = NULL;
+
+        index = poppler_index_iter_new (document);
+
+        if (!index) {
+                return;
+        }
+
+        read_toc (index, &toc);
+
+        if (toc) {
+	        if (toc->len > 0) {
+		        tracker_sparql_builder_predicate (metadata, "nfo:tableOfContents");
+		        tracker_sparql_builder_object_unvalidated (metadata, toc->str);
+	        }
+
+	        g_string_free (toc, TRUE);
+        }
+}
+
 static void
 insert_keywords (TrackerSparqlBuilder *metadata,
                  gchar                *keywords)
@@ -119,6 +248,7 @@ extract_content (PopplerDocument *document,
 
 		g_free (text);
 		g_free (t);
+		g_object_unref (page);
 	}
 
 	return g_string_free (string, FALSE);
@@ -165,6 +295,7 @@ write_pdf_data (PDFData               data,
 
 static void
 extract_pdf (const gchar          *uri,
+             TrackerSparqlBuilder *preupdate,
              TrackerSparqlBuilder *metadata)
 {
 	TrackerFTSConfig *fts_config;
@@ -183,11 +314,20 @@ extract_pdf (const gchar          *uri,
 	document = poppler_document_new_from_file (uri, NULL, &error);
 
 	if (error) {
-		g_warning ("Couldn't create PopplerDocument from uri:'%s', %s",
-		           uri,
-		           error->message ? error->message : "no error given");
-		g_error_free (error);
+		if (error->code == POPPLER_ERROR_ENCRYPTED) {
+			tracker_sparql_builder_predicate (metadata, "a");
+			tracker_sparql_builder_object (metadata, "nfo:PaginatedTextDocument");
 
+			tracker_sparql_builder_predicate (metadata, "nfo:isContentEncrypted");
+			tracker_sparql_builder_object_boolean (metadata, TRUE);
+			return;
+		} else {
+			g_warning ("Couldn't create PopplerDocument from uri:'%s', %s",
+			           uri,
+			           error->message ? error->message : "no error given");
+		}
+
+		g_error_free (error);
 		return;
 	}
 
@@ -198,7 +338,6 @@ extract_pdf (const gchar          *uri,
 		return;
 	}
 
-	tracker_sparql_builder_subject_iri (metadata, uri);
 	tracker_sparql_builder_predicate (metadata, "a");
 	tracker_sparql_builder_object (metadata, "nfo:PaginatedTextDocument");
 
@@ -220,13 +359,13 @@ extract_pdf (const gchar          *uri,
 	}
 
 	if (xml) {
-		tracker_read_xmp (xml, strlen (xml), uri, &xd);
+		tracker_xmp_read (xml, strlen (xml), uri, &xd);
 		g_free (xml);
 		xml = NULL;
 
-		md.title = tracker_coalesce (3, pd.title, xd.title, xd.Title, xd.PDFtitle);
+		md.title = tracker_coalesce (3, pd.title, xd.title, xd.title2, xd.pdf_title);
 		md.subject = tracker_coalesce (2, pd.subject, xd.subject);
-		md.date = tracker_coalesce (3, pd.creation_date, xd.date, xd.DateTimeOriginal);
+		md.date = tracker_coalesce (3, pd.creation_date, xd.date, xd.time_original);
 		md.author = tracker_coalesce (2, pd.author, xd.creator);
 
 		write_pdf_data (md, metadata);
@@ -236,9 +375,9 @@ extract_pdf (const gchar          *uri,
 			g_free (xd.keywords);
 		}
 
-		if (xd.PDFkeywords) {
-			insert_keywords (metadata, xd.PDFkeywords);
-			g_free (xd.PDFkeywords);
+		if (xd.pdf_keywords) {
+			insert_keywords (metadata, xd.pdf_keywords);
+			g_free (xd.pdf_keywords);
 		}
 
 		if (xd.publisher) {
@@ -300,16 +439,16 @@ extract_pdf (const gchar          *uri,
 			g_free (xd.license);
 		}
 
-		if (xd.Make || xd.Model) {
+		if (xd.make || xd.model) {
 			gchar *camera;
 
-			if ((xd.Make == NULL || xd.Model == NULL) ||
-			    (xd.Make && xd.Model && strstr (xd.Model, xd.Make) == NULL)) {
-				camera = tracker_merge (" ", 2, xd.Make, xd.Model);
+			if ((xd.make == NULL || xd.model == NULL) ||
+			    (xd.make && xd.model && strstr (xd.model, xd.make) == NULL)) {
+				camera = tracker_merge (" ", 2, xd.make, xd.model);
 			} else {
-				camera = g_strdup (xd.Model);
-				g_free (xd.Model);
-				g_free (xd.Make);
+				camera = g_strdup (xd.model);
+				g_free (xd.model);
+				g_free (xd.make);
 			}
 
 			tracker_sparql_builder_predicate (metadata, "nmm:camera");
@@ -317,10 +456,10 @@ extract_pdf (const gchar          *uri,
 			g_free (camera);
 		}
 
-		if (xd.Orientation) {
+		if (xd.orientation) {
 			tracker_sparql_builder_predicate (metadata, "nfo:orientation");
-			tracker_sparql_builder_object (metadata, xd.Orientation);
-			g_free (xd.Orientation);
+			tracker_sparql_builder_object (metadata, xd.orientation);
+			g_free (xd.orientation);
 		}
 
 		if (xd.rights) {
@@ -329,42 +468,42 @@ extract_pdf (const gchar          *uri,
 			g_free (xd.rights);
 		}
 
-		if (xd.WhiteBalance) {
+		if (xd.white_balance) {
 			tracker_sparql_builder_predicate (metadata, "nmm:whiteBalance");
-			tracker_sparql_builder_object (metadata, xd.WhiteBalance);
-			g_free (xd.WhiteBalance);
+			tracker_sparql_builder_object (metadata, xd.white_balance);
+			g_free (xd.white_balance);
 		}
 
-		if (xd.FNumber) {
+		if (xd.fnumber) {
 			gdouble value;
 
-			value = g_strtod (xd.FNumber, NULL);
+			value = g_strtod (xd.fnumber, NULL);
 			tracker_sparql_builder_predicate (metadata, "nmm:fnumber");
 			tracker_sparql_builder_object_double (metadata, value);
-			g_free (xd.FNumber);
+			g_free (xd.fnumber);
 		}
 
-		if (xd.Flash) {
+		if (xd.flash) {
 			tracker_sparql_builder_predicate (metadata, "nmm:flash");
-			tracker_sparql_builder_object (metadata, xd.Flash);
-			g_free (xd.Flash);
+			tracker_sparql_builder_object (metadata, xd.flash);
+			g_free (xd.flash);
 		}
 
-		if (xd.FocalLength) {
+		if (xd.focal_length) {
 			gdouble value;
 
-			value = g_strtod (xd.FocalLength, NULL);
+			value = g_strtod (xd.focal_length, NULL);
 			tracker_sparql_builder_predicate (metadata, "nmm:focalLength");
 			tracker_sparql_builder_object_double (metadata, value);
-			g_free (xd.FocalLength);
+			g_free (xd.focal_length);
 		}
 
 		/* Question: Shouldn't xd.Artist be merged with md.author instead? */
 
-		if (xd.Artist || xd.contributor) {
+		if (xd.artist || xd.contributor) {
 			gchar *artist;
 
-			artist = tracker_coalesce (2, xd.Artist, xd.contributor);
+			artist = tracker_coalesce (2, xd.artist, xd.contributor);
 			tracker_sparql_builder_predicate (metadata, "nco:contributor");
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
@@ -375,22 +514,22 @@ extract_pdf (const gchar          *uri,
 			g_free (artist);
 		}
 
-		if (xd.ExposureTime) {
+		if (xd.exposure_time) {
 			gdouble value;
 
-			value = g_strtod (xd.ExposureTime, NULL);
+			value = g_strtod (xd.exposure_time, NULL);
 			tracker_sparql_builder_predicate (metadata, "nmm:exposureTime");
 			tracker_sparql_builder_object_double (metadata, value);
-			g_free (xd.ExposureTime);
+			g_free (xd.exposure_time);
 		}
 
-		if (xd.ISOSpeedRatings) {
+		if (xd.iso_speed_ratings) {
 			gdouble value;
 
-			value = g_strtod (xd.ISOSpeedRatings, NULL);
+			value = g_strtod (xd.iso_speed_ratings, NULL);
 			tracker_sparql_builder_predicate (metadata, "nmm:isoSpeed");
 			tracker_sparql_builder_object_double (metadata, value);
-			g_free (xd.ISOSpeedRatings);
+			g_free (xd.iso_speed_ratings);
 		}
 
 		if (xd.description) {
@@ -399,45 +538,48 @@ extract_pdf (const gchar          *uri,
 			g_free (xd.description);
 		}
 
-		if (xd.MeteringMode) {
+		if (xd.metering_mode) {
 			tracker_sparql_builder_predicate (metadata, "nmm:meteringMode");
-			tracker_sparql_builder_object (metadata, xd.MeteringMode);
-			g_free (xd.MeteringMode);
+			tracker_sparql_builder_object (metadata, xd.metering_mode);
+			g_free (xd.metering_mode);
 		}
 
-		if (xd.Address || xd.Country || xd.City) {
+		if (xd.address || xd.country || xd.city) {
 			tracker_sparql_builder_predicate (metadata, "mlo:location");
 	
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
 			tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
 	
-			if (xd.Address) {
+			if (xd.address) {
 				tracker_sparql_builder_predicate (metadata, "mlo:address");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.Address);
-				g_free (xd.Address);
+				tracker_sparql_builder_object_unvalidated (metadata, xd.address);
+				g_free (xd.address);
 			}
 	
-			if (xd.State) {
+			if (xd.state) {
 				tracker_sparql_builder_predicate (metadata, "mlo:state");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.State);
-				g_free (xd.State);
+				tracker_sparql_builder_object_unvalidated (metadata, xd.state);
+				g_free (xd.state);
 			}
 	
-			if (xd.City) {
+			if (xd.city) {
 				tracker_sparql_builder_predicate (metadata, "mlo:city");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.City);
-				g_free (xd.City);
+				tracker_sparql_builder_object_unvalidated (metadata, xd.city);
+				g_free (xd.city);
 			}
 	
-			if (xd.Country) {
+			if (xd.country) {
 				tracker_sparql_builder_predicate (metadata, "mlo:country");
-				tracker_sparql_builder_object_unvalidated (metadata, xd.Country);
-				g_free (xd.Country);
+				tracker_sparql_builder_object_unvalidated (metadata, xd.country);
+				g_free (xd.country);
 			}
 		
 			tracker_sparql_builder_object_blank_close (metadata);
 		}
+
+		/* PDF keywords aren't used ATM */
+		g_free (pd.keywords);
 	} else {
 		/* So if we are here we have NO XMP data and we just
 		 * write what we know from Poppler.
@@ -458,11 +600,13 @@ extract_pdf (const gchar          *uri,
 		g_free (content);
 	}
 
+        read_outline (document, metadata);
+
 	g_object_unref (document);
 }
 
 TrackerExtractData *
-tracker_get_extract_data (void)
+tracker_extract_get_data (void)
 {
 	return data;
 }
